@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2019 Open Source Applications Foundation.
+ * Copyright (c) 2004-2024 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -211,7 +211,7 @@ PyObject *wrap_##name(icuClass *object, int flags)                        \
     Py_RETURN_NONE;                                                       \
 }
 
-#define DECLARE_BY_VALUE_TYPE(name, t_name, base, icuClass, init)     \
+#define DECLARE_DEALLOC_TYPE(name, t_name, base, icuClass, init)      \
 void t_name##_dealloc(t_name *self)                                   \
 {                                                                     \
     if (self->flags & T_OWNED)                                        \
@@ -219,7 +219,10 @@ void t_name##_dealloc(t_name *self)                                   \
     self->object = NULL;                                              \
     Py_TYPE(self)->tp_free((PyObject *) self);                        \
 }                                                                     \
-DECLARE_TYPE(name, t_name, base, icuClass, init, t_name##_dealloc)    \
+DECLARE_TYPE(name, t_name, base, icuClass, init, t_name##_dealloc)
+
+#define DECLARE_BY_VALUE_TYPE(name, t_name, base, icuClass, init)     \
+DECLARE_DEALLOC_TYPE(name, t_name, base, icuClass, init)              \
 PyObject *wrap_##name(icuClass &object)                               \
 {                                                                     \
     return wrap_##name(new icuClass(std::move(object)), T_OWNED);     \
@@ -416,13 +419,13 @@ PyTypeObject name##Type_ = {                                            \
     }
 
 
-#define DEFINE_RICHCMP(name, t_name) \
+#define DEFINE_RICHCMP__ARG__(name, t_name) \
     static PyObject *t_name ## _richcmp(t_name *self,                     \
                                         PyObject *arg, int op)            \
     {                                                                     \
         int b = 0;                                                        \
         name *object;                                                     \
-        if (!parseArg(arg, "P", TYPE_CLASSID(name), &object))             \
+        if (!parseArg(arg, arg::P<name>(TYPE_CLASSID(name), &object)))    \
         {                                                                 \
             switch (op) {                                                 \
               case Py_EQ:                                                 \
